"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _reactRouter = require("react-router");

var _crypto = _interopRequireDefault(require("crypto"));

var _fs = _interopRequireDefault(require("fs"));

var _renderApp = _interopRequireDefault(require("./renderApp"));

var _createStore = _interopRequireDefault(require("../createStore"));

var _prefetchData = _interopRequireDefault(require("./prefetchData"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _default = function _default(req, res, options) {
  var routes = options.routes,
      reducers = options.reducers,
      initialState = options.initialState,
      routesFetchersMap = options.routesFetchersMap,
      ignoredPathsRegex = options.ignoredPathsRegex;

  var onError = options.onError || function () {};

  var log = options.customLog || function (message) {
    console.log(message);
  };

  (0, _reactRouter.match)({
    routes: routes,
    location: req.url
  }, function (error, redirectLocation, renderProps) {
    if (error) {
      onError(error);
      return res.status(200).end('');
    }

    if (redirectLocation) {
      return res.status(301).redirect(redirectLocation.pathname);
    }

    if (!renderProps) {
      return res.status(404).end('Not found'); // TODO: render 404 on client?
    }

    var shouldUseCache = Boolean(options.cachePath);
    var cachedPagePath;

    if (shouldUseCache) {
      var filename = _crypto["default"].createHash('md5').update(req.url).digest('hex');

      cachedPagePath = "".concat(options.cachePath, "/").concat(filename, ".html");

      if (_fs["default"].existsSync(cachedPagePath)) {
        var html = _fs["default"].readFileSync(cachedPagePath);

        res.setHeader('Content-Type', 'text/html');
        return res.send(html);
      }
    }

    var store = (0, _createStore["default"])(reducers, initialState);
    var prefetcherOptions = {
      ignoredPathsRegex: ignoredPathsRegex,
      log: log
    };
    (0, _prefetchData["default"])(routesFetchersMap, renderProps.location.pathname, req.query, prefetcherOptions, store).then(function () {
      return (0, _renderApp["default"])(renderProps, store, options);
    }).then(function (html) {
      if (shouldUseCache) {
        _fs["default"].writeFileSync(cachedPagePath, html);
      }

      res.setHeader('Content-Type', 'text/html');
      return res.send(html);
    })["catch"](function (err) {
      onError(err);
      return res.status(200).end('');
    });
  });
};

exports["default"] = _default;